/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_ARRAYLIST_INL
#define __TOOLBOX_ARRAYLIST_INL


// --- Class ArrayList ---

template <class ELEMENTTYPE> ArrayList<ELEMENTTYPE>::ArrayList()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Init(_TB_AL_DEFAULTINITSIZE, _TB_AL_DEFAULTMINIMUMFILLRATIO, _TB_AL_DEFAULTDESIREDFILLRATIO);
}

template <class ELEMENTTYPE> ArrayList<ELEMENTTYPE>::ArrayList(long initSize)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Init(initSize, _TB_AL_DEFAULTMINIMUMFILLRATIO, _TB_AL_DEFAULTDESIREDFILLRATIO);
}

template <class ELEMENTTYPE> ArrayList<ELEMENTTYPE>::ArrayList(long initSize,
	int minimumFillRatio, int desiredFillRatio)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Init(initMaxCount, minimumFillRatio, desiredFillRatio);
}

template <class ELEMENTTYPE> ArrayList<ELEMENTTYPE>::~ArrayList()
{
	ASSERTION_THIS;

	DeleteAll();
	if (Elements) {delete Elements; Elements=NULL;}

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::Append(ELEMENTTYPE * Element)
{
	ASSERTION_THIS;

	CheckSize(Count + 1);
	Elements[Count++] = Element;
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::AppendAll(const ArrayList<ELEMENTTYPE> * Elements)
{
	ASSERTION_THIS;
	ASSERTION(Elements != NULL);

	int elementsCount = Elements->GetCount();
	CheckSize(Count + elementsCount);

	for (int i=0; i<elementsCount; i++)
		this->Elements[Count++] = Elements->Get(i);
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::CheckSize(int newSize)
{
	ASSERTION_THIS;
	int desiredSize = 100 * newSize / DesiredFillRatio;

	if (Size > InitSize && GetFillRatio() < MinimumFillRatio)
	{
		if (GetFillRatio() >= MinimumFillRatio)
		{
			if (newSize >= Size)
				Resize(desiredSize > newSize ? desiredSize : newSize + 1);
		}
		else
		{
			if (desiredSize >= InitSize)
				Resize(desiredSize > newSize ? desiredSize : newSize + 1);
			else
				Resize(InitSize > newSize ? InitSize : newSize + 1);
		}
	}
	else
		if (newSize >= Size)
			Resize(desiredSize > newSize ? desiredSize : newSize + 1);
}

template <class ELEMENTTYPE> int ArrayList<ELEMENTTYPE>::Compare(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2) const
{
	ASSERTION_THIS;

	return Element1 == Element2 ? 0 : (Element1 > Element2 ? 1 : -1);
}

template <class ELEMENTTYPE> bool ArrayList<ELEMENTTYPE>::Contains(const ELEMENTTYPE * Element) const
{
	ASSERTION_THIS;
	return Contains(Element, NULL);
}

template <class ELEMENTTYPE> bool ArrayList<ELEMENTTYPE>::Contains(const ELEMENTTYPE * Element,
	int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const
{
	ASSERTION_THIS;

	for (int i=0; i<Count; i++)
		if ((compare ? compare(Elements[i], Element) : Compare(Elements[i], Element)) == 0)
			return true;

	return false;
}

template <class ELEMENTTYPE> bool ArrayList<ELEMENTTYPE>::Delete(long nr)
{
	ASSERTION_THIS;

	ELEMENTTYPE * DeleteElement;
	if ((DeleteElement = ArrayList<ELEMENTTYPE>::Unlink(nr)) != NULL)
	{
		delete (ELEMENTTYPE *) DeleteElement;
		return true;
	}
	else
		return false;
}

template <class ELEMENTTYPE> bool ArrayList<ELEMENTTYPE>::Delete(long from, long count)
{
	ASSERTION_THIS;

	if (from < 0 || count < 0 || from + count > Count)
		return false;

	for (int i=from; i<from + count; i++)
		delete Elements[i];

	memmove(&Elements[from], &Elements[from + count], sizeof(ELEMENTTYPE *) * (Count - from - count));
	CheckSize(Count - count);

	Count -= count;

	return true;
}

template <class ELEMENTTYPE> bool ArrayList<ELEMENTTYPE>::Delete(ELEMENTTYPE * Element)
{
	ASSERTION_THIS;

	ELEMENTTYPE * DeleteElement;
	if ((DeleteElement=ArrayList<ELEMENTTYPE>::Unlink(Element)) != NULL)
	{
		delete (ELEMENTTYPE *) DeleteElement;
		return true;
	}
	else
		return false;
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::DeleteAll()
{
	ASSERTION_THIS;

	for (int i=0; i<Count; i++)
		delete Elements[i];

	CheckSize(0);
	Count = 0;
}

template <class ELEMENTTYPE> inline bool ArrayList<ELEMENTTYPE>::DeleteFirst()
{
	ASSERTION_THIS;
	return Delete((long) 0);
}

template <class ELEMENTTYPE> bool ArrayList<ELEMENTTYPE>::DeleteLast()
{
	ASSERTION_THIS;

	ELEMENTTYPE * DeleteElement;

	if ((DeleteElement = UnlinkLast()) != NULL)
	{
		delete (ELEMENTTYPE *) DeleteElement;
		return true;
	}
	else
		return false;
}

template <class ELEMENTTYPE> int ArrayList<ELEMENTTYPE>::Find(const ELEMENTTYPE * Element) const
{
	ASSERTION_THIS;
	return Find(Element, NULL);
}

template <class ELEMENTTYPE> int ArrayList<ELEMENTTYPE>::Find(const ELEMENTTYPE * Element,
	int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const
{
	ASSERTION_THIS;

	for (int i=0; i<Count; i++)
		if ((compare ? compare(Elements[i], Element) : Compare(Elements[i], Element)) == 0)
			return i;

	return -1;
}

template <class ELEMENTTYPE> ELEMENTTYPE * ArrayList<ELEMENTTYPE>::Get(long nr) const
{
	ASSERTION_THIS;

	if (nr < 0 || nr >= Count)
		return NULL;

	return Elements[nr];
}

template <class ELEMENTTYPE> inline long ArrayList<ELEMENTTYPE>::GetCount() const
{
	ASSERTION_THIS;
	return Count;
}

template <class ELEMENTTYPE> inline int ArrayList<ELEMENTTYPE>::GetDesiredFillRatio() const
{
	ASSERTION_THIS;
	return DesiredFillRatio;
}

template <class ELEMENTTYPE> inline int ArrayList<ELEMENTTYPE>::GetFillRatio() const
{
	ASSERTION_THIS;
	return Count * 100 / Size;
}

template <class ELEMENTTYPE> ELEMENTTYPE * ArrayList<ELEMENTTYPE>::GetFirst() const
{
	ASSERTION_THIS;

	if (Count <= 0)
		return NULL;

	return Elements[0];
}

template <class ELEMENTTYPE> ELEMENTTYPE * ArrayList<ELEMENTTYPE>::GetLast() const
{
	ASSERTION_THIS;

	if (Count <= 0)
		return NULL;

	return Elements[Count-1];
}

template <class ELEMENTTYPE> inline int ArrayList<ELEMENTTYPE>::GetMinimumFillRatio() const
{
	ASSERTION_THIS;
	return MinimumFillRatio;
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::Init(long initSize,
	int minimumFillRatio, int desiredFillRatio)
{
	ASSERTION_THIS;

	Count = 0;
	DesiredFillRatio = 0;
	Elements = NULL;
	InitSize = 0;
	MinimumFillRatio = 0;
	Size = 0;

	if (initSize < 8)
		initSize = 8;

	if (minimumFillRatio < 0 || desiredFillRatio <= minimumFillRatio
		|| desiredFillRatio >= 100)
	{
		minimumFillRatio = _TB_AL_DEFAULTMINIMUMFILLRATIO;
		desiredFillRatio = _TB_AL_DEFAULTDESIREDFILLRATIO;
	}

	MinimumFillRatio = minimumFillRatio;
	DesiredFillRatio = desiredFillRatio;
	InitSize = initSize;

	Resize(initSize);
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::Insert(ELEMENTTYPE * Element, long nr)
{
	ASSERTION_THIS;

	if (nr < 0 || nr > Count)
		return;

	CheckSize(Count + 1);

	memmove(&Elements[nr+1], &Elements[nr], sizeof(ELEMENTTYPE *) * (Count - nr));
	Count++;
	Elements[nr] = Element;
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::InsertAll(const ArrayList<ELEMENTTYPE> * Elements, long nr)
{
	ASSERTION_THIS;
	ASSERTION(Elements != NULL);

	if (nr < 0 || nr > Count)
		return;

	int elementsCount = Elements->GetCount();
	CheckSize(Count + elementsCount);

	memmove(&this->Elements[nr + elementsCount], &this->Elements[nr],
		sizeof(ELEMENTTYPE *) * (Count-nr));
	for (int i=0; i<elementsCount; i++)
		this->Elements[nr + i] = Elements->Get(i);

	Count += elementsCount;
}

#ifdef _TOOLBOX_TEST
	template <class ELEMENTTYPE> int ArrayList<ELEMENTTYPE>::InstanceCount = 0;
#endif

template <class ELEMENTTYPE> inline bool ArrayList<ELEMENTTYPE>::IsEmpty() const
{
	ASSERTION_THIS;
	return Count <= 0;
}

template <class ELEMENTTYPE> inline void ArrayList<ELEMENTTYPE>::Prepend(ELEMENTTYPE * Element)
{
	ASSERTION_THIS;
	Insert(Element, 0);
}

template <class ELEMENTTYPE> inline void ArrayList<ELEMENTTYPE>::PrependAll(const ArrayList<ELEMENTTYPE> * Elements)
{
	ASSERTION_THIS;
	ASSERTION(Elements != NULL);
	InsertAll(Elements, 0);
}

template <class ELEMENTTYPE> ELEMENTTYPE * ArrayList<ELEMENTTYPE>::Replace(ELEMENTTYPE * NewElement, long nr)
{
	ASSERTION_THIS;

	if (nr < 0 || nr >= Count)
		return NULL;

	ELEMENTTYPE * OldElement = Elements[nr];
	Elements[nr] = NewElement;
	return OldElement;
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::Resize(long newSize)
{
	ASSERTION_THIS;

	ELEMENTTYPE ** NewElements;
	if ((NewElements = new ELEMENTTYPE * [newSize]) == NULL)
		throw EOutOfMemory::Get();

	if (Elements)
	{
		memmove(NewElements, Elements, sizeof(ELEMENTTYPE *) * Count);
		delete Elements;
	}

	Elements = NewElements;
	Size = newSize;
}

#ifdef _TOOLBOX_TEST
	template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::RunTestSuite(int * performedTests, int * failedTests)
	{
		ArrayList<TestObject> * TestArray = new ArrayList<TestObject>;

		if (TestObject::GetInstanceCount() != 0) {printf("Test Array[0] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->GetCount() != 0) {printf("Test Array[1] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestObject * Object1 = new TestObject;
		Object1->Number = 1;
		TestArray->Append(Object1);
		TestObject * Object2 = new TestObject;
		Object2->Number = 2;
		TestArray->Append(Object2);

		if (TestObject::GetInstanceCount() != 2) {printf("Test Array[2] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->GetCount() != 2) {printf("Test Array[3] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Get(0)->Number != 1) {printf("Test Array[4] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Get(1)->Number != 2) {printf("Test Array[5] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestObject * Object3 = new TestObject;
		Object3->Number = 3;

		if (TestArray->Find(Object1) != 0) {printf("Test Array[5.1] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Find(Object2) != 1) {printf("Test Array[5.2] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Find(Object3) != -1) {printf("Test Array[5.3] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestArray->Unlink((long) 0);
		if (TestArray->Get(0)->Number != 2) {printf("Test Array[6] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		TestArray->Delete(Object1);
		if (TestArray->GetCount() != 1) {printf("Test Array[7] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		TestArray->Delete(Object2);
		if (TestArray->GetCount() != 0) {printf("Test Array[8] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		delete Object1;

		TestArray->Append(Object3);
		TestObject * Object4 = new TestObject;
		Object4->Number = 4;
		TestArray->Prepend(Object4);
		TestObject * Object5= new TestObject;
		Object5->Number = 5;
		TestArray->Insert(Object5, 1);

		if (TestArray->GetCount() != 3) {printf("Test Array[9] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Get(0)->Number != 4) {printf("Test Array[10] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Get(1)->Number != 5) {printf("Test Array[11] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Get(2)->Number != 3) {printf("Test Array[12] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestArray->DeleteFirst();
		if (TestArray->Get(0)->Number != 5) {printf("Test Array[13] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Get(1)->Number != 3) {printf("Test Array[14] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		TestArray->DeleteLast();
		if (TestArray->Get(0)->Number != 5) {printf("Test Array[15] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->GetCount() != 1) {printf("Test Array[16] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->IsEmpty()) {printf("Test Array[17] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestArray->DeleteAll();
		if (!TestArray->IsEmpty()) {printf("Test Array[18] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->GetCount() != 0) {printf("Test Array[19] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestObject::GetInstanceCount() != 0) {printf("Test Array[20] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestObject * Object6 = new TestObject;
		Object6->Number = 6;
		TestArray->Append(Object6);
		TestObject * Object7 = new TestObject;
		Object7->Number = 7;
		TestArray->Append(Object7);
		TestObject * Object8= new TestObject;
		Object8->Number = 8;
		TestArray->Append(Object8);

		TestArray->UnlinkFirst();
		if (TestArray->Get(0)->Number != 7) {printf("Test Array[21] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->Get(1)->Number != 8) {printf("Test Array[22] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		TestArray->UnlinkLast();
		if (TestArray->Get(0)->Number != 7) {printf("Test Array[23] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestArray->UnlinkAll();
		if (TestArray->GetCount() != 0) {printf("Test Array[24] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		delete Object6;
		delete Object7;
		delete Object8;

		if (TestObject::GetInstanceCount() != 0) {printf("Test Array[25] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestObject * Object9= new TestObject;
		Object9->Number = 9;
		TestArray->Append(Object9);
		TestObject * Object10= new TestObject;
		Object10->Number = 10;
		TestArray->Append(Object10);
		TestObject * Object11= new TestObject;
		Object11->Number = 11;
		TestArray->Append(Object11);
		TestObject * Object12= new TestObject;
		Object12->Number = 12;
		TestArray->Append(Object12);
		TestObject * Object13= new TestObject;
		Object13->Number = 13;
		TestArray->Append(Object13);
		TestObject * Object14= new TestObject;
		Object14->Number = 14;
		TestArray->Append(Object14);
		TestObject * Object15= new TestObject;
		Object15->Number = 15;
		TestArray->Append(Object15);
		TestObject * Object16= new TestObject;
		Object16->Number = 16;
		TestArray->Append(Object16);
		TestObject * Object17= new TestObject;
		Object17->Number = 17;
		TestArray->Append(Object17);
		TestObject * Object18= new TestObject;
		Object18->Number = 18;
		TestArray->Append(Object18);
		TestObject * Object19= new TestObject;
		Object19->Number = 19;
		TestArray->Append(Object19);
		TestObject * Object20= new TestObject;
		Object20->Number = 20;
		TestArray->Append(Object20);
		TestObject * Object21= new TestObject;

		if (TestArray->Size != 16) {printf("Test Array[26] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		Object21->Number = 21;
		TestArray->Append(Object21);
		TestObject * Object22= new TestObject;
		Object22->Number = 22;
		TestArray->Append(Object22);
		TestObject * Object23= new TestObject;
		Object23->Number = 23;
		TestArray->Append(Object23);
		TestObject * Object24= new TestObject;
		Object24->Number = 24;
		TestArray->Append(Object24);
		TestObject * Object25= new TestObject;
		Object25->Number = 25;
		TestArray->Append(Object25);
		TestObject * Object26= new TestObject;
		Object26->Number = 26;
		TestArray->Append(Object26);
		TestObject * Object27= new TestObject;
		Object27->Number = 27;
		TestArray->Append(Object27);
		TestObject * Object28= new TestObject;
		Object28->Number = 28;
		TestArray->Append(Object28);

		if (TestArray->Size != 26) {printf("Test Array[27] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->GetCount() != 20) {printf("Test Array[28] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		if (TestArray->Get(0)->Number != 9
			|| TestArray->Get(1)->Number != 10
			|| TestArray->Get(2)->Number != 11
			|| TestArray->Get(3)->Number != 12
			|| TestArray->Get(4)->Number != 13
			|| TestArray->Get(5)->Number != 14
			|| TestArray->Get(6)->Number != 15
			|| TestArray->Get(7)->Number != 16
			|| TestArray->Get(8)->Number != 17
			|| TestArray->Get(9)->Number != 18
			|| TestArray->Get(10)->Number != 19
			|| TestArray->Get(11)->Number != 20
			|| TestArray->Get(12)->Number != 21
			|| TestArray->Get(13)->Number != 22
			|| TestArray->Get(14)->Number != 23
			|| TestArray->Get(15)->Number != 24
			|| TestArray->Get(16)->Number != 25
			|| TestArray->Get(17)->Number != 26
			|| TestArray->Get(18)->Number != 27
			|| TestArray->Get(19)->Number != 28)
			{printf("Test Array[29] failed!\n"); (*failedTests)++;}

		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();
		TestArray->DeleteLast();

		if (TestArray->Size != 26) {printf("Test Array[30] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		TestArray->DeleteLast();

		if (TestArray->Size != 26) {printf("Test Array[31] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
		if (TestArray->GetCount() != 7) {printf("Test Array[32] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		if (TestArray->Get(0)->Number != 9
			|| TestArray->Get(1)->Number != 10
			|| TestArray->Get(2)->Number != 11
			|| TestArray->Get(3)->Number != 12
			|| TestArray->Get(4)->Number != 13
			|| TestArray->Get(5)->Number != 14
			|| TestArray->Get(6)->Number != 15)
			{printf("Test Array[33] failed!\n"); (*failedTests)++;}

		TestArray->DeleteLast();

		if (TestArray->Size != 16) {printf("Test Array[34] failed!\n"); (*failedTests)++;}
		(*performedTests)++;

		delete TestArray;
		if (ArrayList::InstanceCount != 0) {printf("Test Array[35] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
	}
#endif

template <class ELEMENTTYPE> inline void ArrayList<ELEMENTTYPE>::Sort()
{
	ASSERTION_THIS;
	Sort(NULL);
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::Sort(int (* compare)(const ELEMENTTYPE * Element1,
	const ELEMENTTYPE * Element2))
{
	ASSERTION_THIS;

	for (int firstNr=0; firstNr<Count-1; firstNr++)
	{
		int minNr = firstNr;
		for (int searchNr=firstNr+1; searchNr<Count; searchNr++)
			if ((compare ? compare(Elements[searchNr], Elements[minNr]) : Compare(Elements[searchNr], Elements[minNr])) < 0)
				minNr = searchNr;

		ELEMENTTYPE * Memorize = Elements[minNr];
		Elements[minNr] = Elements[firstNr];
		Elements[firstNr] = Memorize;
	}
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::Shuffle()
{
	ASSERTION_THIS;

	for (int i = 0; i < Count - 1; i++)
	{
		int pos = (int) (rand() * 1.0 * (Count - i) / RAND_MAX) + i;
		ELEMENTTYPE * Memorize = Elements[pos];
		Elements[pos] = Elements[i];
		Elements[i] = Memorize;
	}
}

template <class ELEMENTTYPE> ELEMENTTYPE * ArrayList<ELEMENTTYPE>::Unlink(long nr)
{
	ASSERTION_THIS;

	if (nr < 0 || nr >= Count)
		return NULL;

	ELEMENTTYPE * OldElement = Elements[nr];
	memmove(&Elements[nr], &Elements[nr+1], sizeof(ELEMENTTYPE *) * (Count - nr - 1));

	CheckSize(Count - 1);
	Count--;

	return OldElement;
}

template <class ELEMENTTYPE> bool ArrayList<ELEMENTTYPE>::Unlink(long from, long count)
{
	ASSERTION_THIS;

	if (from < 0 || count < 0 || from + count > Count)
		return false;

	memmove(&Elements[from], &Elements[from + count], sizeof(ELEMENTTYPE *) * (Count - from - count));

	CheckSize(Count - count);
	Count -= count;

	return true;
}

template <class ELEMENTTYPE> ELEMENTTYPE * ArrayList<ELEMENTTYPE>::Unlink(ELEMENTTYPE * Element)
{
	ASSERTION_THIS;

	for (int i=0; i<Count; i++)
		if (Elements[i] == Element)
			return Unlink(i);

	return NULL;
}

template <class ELEMENTTYPE> void ArrayList<ELEMENTTYPE>::UnlinkAll()
{
	ASSERTION_THIS;

	CheckSize(0);
	Count = 0;
}

template <class ELEMENTTYPE> inline ELEMENTTYPE * ArrayList<ELEMENTTYPE>::UnlinkFirst()
{
	ASSERTION_THIS;
	return Unlink((long) 0);
}

template <class ELEMENTTYPE> ELEMENTTYPE * ArrayList<ELEMENTTYPE>::UnlinkLast()
{
	ASSERTION_THIS;

	if (Count <= 0)
		return NULL;

	ELEMENTTYPE * OldElement = Elements[Count - 1];

	CheckSize(Count - 1);
	Count--;

	return OldElement;
}


#endif
