/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_DIRECTORY_INL
#define __TOOLBOX_DIRECTORY_INL


// --- Class Directory ---

inline void Directory::AppendEntry(DirectoryEntry * Entry)
{
	ASSERTION_THIS;
	Entries->Append(Entry);
}

inline void Directory::DeleteAll()
{
	ASSERTION_THIS;
	Entries->DeleteAll();
}

inline bool Directory::DeleteEntry(long nr)
{
	ASSERTION_THIS;
	return Entries->Delete(nr);
}

inline bool Directory::DeleteEntry(DirectoryEntry * Entry)
{
	ASSERTION_THIS;
	return Entries->Delete(Entry);
}

inline bool Directory::DeleteFirstEntry()
{
	ASSERTION_THIS;
	return Entries->DeleteFirst();
}

inline bool Directory::DeleteLastEntry()
{
	ASSERTION_THIS;
	return Entries->DeleteLast();
}

inline DirectoryEntry * Directory::GetEntry(long nr)
{
	ASSERTION_THIS;
	return Entries->Get(nr);
}

inline int Directory::GetEntryCount()
{
	ASSERTION_THIS;
	return Entries->GetCount();
}

inline DirectoryEntry * Directory::GetFirstEntry()
{
	ASSERTION_THIS;
	return Entries->GetFirst();
}

inline DirectoryEntry * Directory::GetLastEntry()
{
	ASSERTION_THIS;
	return Entries->GetLast();
}

inline void Directory::InsertEntry(DirectoryEntry * Entry, long nr)
{
	ASSERTION_THIS;
	Entries->Insert(Entry, nr);
}

inline bool Directory::IsEmpty()
{
	ASSERTION_THIS;
	return Entries->IsEmpty();
}

inline void Directory::PrependEntry(DirectoryEntry * Entry)
{
	ASSERTION_THIS;
	Entries->Prepend(Entry);
}

inline DirectoryEntry * Directory::UnlinkEntry(long nr)
{
	ASSERTION_THIS;
	return Entries->Unlink(nr);
}

inline DirectoryEntry * Directory::UnlinkEntry(DirectoryEntry * Entry)
{
	ASSERTION_THIS;
	return Entries->Unlink(Entry);
}

inline DirectoryEntry * Directory::UnlinkFirstEntry()
{
	ASSERTION_THIS;
	return Entries->UnlinkFirst();
}

inline DirectoryEntry * Directory::UnlinkLastEntry()
{
	ASSERTION_THIS;
	return Entries->UnlinkLast();
}


// --- Class DirectoryEntry ---

inline bool DirectoryEntry::Exists()
{
	ASSERTION_THIS;
	return ExistsFlag;
}

inline int DirectoryEntry::GetAccessTimeSecond()
{
	ASSERTION_THIS;
	return AccessTime.tm_sec;
}

inline int DirectoryEntry::GetAccessTimeMinute()
{
	ASSERTION_THIS;
	return AccessTime.tm_min;
}

inline int DirectoryEntry::GetAccessTimeHour()
{
	ASSERTION_THIS;
	return AccessTime.tm_hour;
}

inline int DirectoryEntry::GetAccessTimeDay()
{
	ASSERTION_THIS;
	return AccessTime.tm_mday;
}

inline int DirectoryEntry::GetAccessTimeMonth()
{
	ASSERTION_THIS;
	return AccessTime.tm_mon;
}

inline int DirectoryEntry::GetAccessTimeYear()
{
	ASSERTION_THIS;
	return AccessTime.tm_year;
}

inline int DirectoryEntry::GetCreateTimeSecond()
{
	ASSERTION_THIS;
	return CreateTime.tm_sec;
}

inline int DirectoryEntry::GetCreateTimeMinute()
{
	ASSERTION_THIS;
	return CreateTime.tm_min;
}

inline int DirectoryEntry::GetCreateTimeHour()
{
	ASSERTION_THIS;
	return CreateTime.tm_hour;
}

inline int DirectoryEntry::GetCreateTimeDay()
{
	ASSERTION_THIS;
	return CreateTime.tm_mday;
}

inline int DirectoryEntry::GetCreateTimeMonth()
{
	ASSERTION_THIS;
	return CreateTime.tm_mon;
}

inline int DirectoryEntry::GetCreateTimeYear()
{
	ASSERTION_THIS;
	return CreateTime.tm_year;
}

inline long DirectoryEntry::GetFlags()
{
	ASSERTION_THIS;
	return Flags;
}

inline int DirectoryEntry::GetModifyTimeSecond()
{
	ASSERTION_THIS;
	return ModifyTime.tm_sec;
}

inline int DirectoryEntry::GetModifyTimeMinute()
{
	ASSERTION_THIS;
	return ModifyTime.tm_min;
}

inline int DirectoryEntry::GetModifyTimeHour()
{
	ASSERTION_THIS;
	return ModifyTime.tm_hour;
}

inline int DirectoryEntry::GetModifyTimeDay()
{
	ASSERTION_THIS;
	return ModifyTime.tm_mday;
}

inline int DirectoryEntry::GetModifyTimeMonth()
{
	ASSERTION_THIS;
	return ModifyTime.tm_mon;
}

inline int DirectoryEntry::GetModifyTimeYear()
{
	ASSERTION_THIS;
	return ModifyTime.tm_year;
}

inline class DirectoryEntry * DirectoryEntry::GetNextEntry()
{
	ASSERTION_THIS;
	return GetNext();
}

inline Path * DirectoryEntry::GetPath()
{
	ASSERTION_THIS;
	return Path;
}

inline class DirectoryEntry * DirectoryEntry::GetPrevEntry()
{
	ASSERTION_THIS;
	return GetPrev();
}

inline unsigned long DirectoryEntry::GetSize()
{
	ASSERTION_THIS;
	return Size;
}


// --- Class Path ---

inline void Path::AppendElement(const char * elementName)
{
	ASSERTION_THIS;

	PathElement * NewElement;
	if ((NewElement = new PathElement(elementName)) == NULL)
		throw EOutOfMemory::Get();
	Elements->Append(NewElement);
}

inline void Path::AppendElement(PathElement * Element)
{
	ASSERTION_THIS;
	Elements->Append(Element);
}

inline void Path::DeleteAll()
{
	ASSERTION_THIS;
	Elements->DeleteAll();
}

inline bool Path::DeleteElement(long nr)
{
	ASSERTION_THIS;
	return Elements->Delete(nr);
}

inline bool Path::DeleteElement(PathElement * Element)
{
	ASSERTION_THIS;
	return Elements->Delete(Element);
}

inline bool Path::DeleteFirstElement()
{
	ASSERTION_THIS;
	return Elements->DeleteFirst();
}

inline bool Path::DeleteLastElement()
{
	ASSERTION_THIS;
	return Elements->DeleteLast();
}

inline PathElement * Path::GetElement(long nr)
{
	ASSERTION_THIS;
	return Elements->Get(nr);
}

inline int Path::GetElementCount()
{
	ASSERTION_THIS;
	return Elements->GetCount();
}

inline PathElement * Path::GetFirstElement()
{
	ASSERTION_THIS;
	return Elements->GetFirst();
}

inline PathElement * Path::GetLastElement()
{
	ASSERTION_THIS;
	return Elements->GetLast();
}

inline char * Path::GetRoot()
{
	ASSERTION_THIS;
	return Root;
}

inline void Path::InsertElement(const char * elementName, long nr)
{
	ASSERTION_THIS;

	PathElement * NewElement;
	if ((NewElement = new PathElement(elementName)) == NULL)
		throw EOutOfMemory::Get();
	Elements->Insert(NewElement, nr);
}

inline void Path::InsertElement(PathElement * Element, long nr)
{
	ASSERTION_THIS;
	Elements->Insert(Element, nr);
}

inline bool Path::IsAbsolute()
{
	ASSERTION_THIS;
	return Root[0] != 0;
}

inline bool Path::IsEmpty()
{
	ASSERTION_THIS;
	return Elements->IsEmpty();
}

inline bool Path::IsRelative()
{
	ASSERTION_THIS;
	return Root[0] == 0;
}

inline void Path::PrependElement(const char * elementName)
{
	ASSERTION_THIS;

	PathElement * NewElement;
	if ((NewElement = new PathElement(elementName)) == NULL)
		throw EOutOfMemory::Get();
	Elements->Prepend(NewElement);
}

inline void Path::PrependElement(PathElement * Element)
{
	ASSERTION_THIS;
	Elements->Prepend(Element);
}

inline char * Path::ToString()
{
	ASSERTION_THIS;
	return ToString(false);
}

inline PathElement * Path::UnlinkElement(long nr)
{
	ASSERTION_THIS;
	return Elements->Unlink(nr);
}

inline PathElement * Path::UnlinkElement(PathElement * Element)
{
	ASSERTION_THIS;
	return Elements->Unlink(Element);
}

inline PathElement * Path::UnlinkFirstElement()
{
	ASSERTION_THIS;
	return Elements->UnlinkFirst();
}

inline PathElement * Path::UnlinkLastElement()
{
	ASSERTION_THIS;
	return Elements->UnlinkLast();
}


// --- Class PathElement ---

inline char * PathElement::GetName()
{
	ASSERTION_THIS;
	return Name;
}

inline class PathElement * PathElement::GetNextElement()
{
	ASSERTION_THIS;
	return GetNext();
}

inline class PathElement * PathElement::GetPrevElement()
{
	ASSERTION_THIS;
	return GetPrev();
}


#endif
