/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TB_HEADERS_H
#define __TB_HEADERS_H


// Non-component headers, level 0
#include <direct.h>
#include <io.h>
#include <memory.h>
#include <share.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys\stat.h>
#include <sys\types.h>
#include <time.h>

// Non-component headers, level 1
#if defined(_DEBUG) && (defined(_AFX) || defined(_AFXDLL))
	// MFC headers
	#include <afx.h>
	#include <afxcmn.h>
	#include <afxext.h>
	#include <afxpriv.h>
	#include <afxsock.h>
	#include <afxwin.h>
#else
	#ifdef _WINDOWS
		#include <windows.h>
	#endif
#endif

// Non-component headers, level 2
#if defined(D3D_OVERLOADS)
	#include <d3dx9.h>
	#include <dxerr9.h>
#endif


// Class declarations
namespace toolbox
{
	template <class ELEMENTTYPE> class ArrayList;
	class AutoDeleting;
	class Bool;
	class Char;
	class ChecksumCalculator;
	class CommandLine;
	class Config;
	class ConfigParameter;
	class ConfigSection;
	template <class ITEMTYPE> class ContainingList;
	template <class ITEMTYPE> class ContainingListItem;
	template <class ITEMTYPE> class ContainingStack;
	template <class ITEMTYPE> class ContainingStackItem;
	class Directory;
	class DirectoryEntry;
	class Double;
	class EAssertionFailed;
	class EDeviceError;
	class EIOError;
	class EIllegalArgument;
	class EIllegalState;
	class EIndexOutOfBounds;
	class EInternalError;
	class ENoSuchElement;
	class ENotSupported;
	class ENullPointer;
	class Edge;
	class EdgeArray;
	class Exception;
	class Float;
	class Graph;
	template <class VALUETYPE> class HashMap;
	class HashSet;
	template <class VALUETYPE> class ICStringKeyReferenceValueMap;
	template <class VALUETYPE> class ICStringKeyHashMap;
	class ICStringKeyHashSet;
	class Int;
	template <class VALUETYPE> class IntKeyHashMap;
	class IntKeyHashSet;
	template <class VALUETYPE> class IntKeyReferenceValueMap;
	class Logger;
	class LongDouble;
	class LongInt;
	class MD5Hasher;
	class MP3FileInfos;
	class Path;
	class PathElement;
	class Pointer;
	class PointeredList;
	class PointeredListItem;
	class PointeredStack;
	class PointeredStackItem;
	class Reference;
	template <class KEYTYPE, class VALUETYPE> class ReferenceKeyMap;
	template <class ELEMENTTYPE> class ReferenceList;
	template <class VALUETYPE> class ReferenceValueMap;
	class ShortInt;
	class SignedChar;
	class SignedInt;
	class SignedLongInt;
	class SignedShortInt;
	class Simulation;
	class SimulationEndEvent;
	class SimulationEvent;
	class SimulationObject;
	template <class ELEMENTTYPE> class SingleReference;
	class String;
	class StringKeyHashSet;
	template <class VALUETYPE> class StringKeyHashMap;
	template <class VALUETYPE> class StringKeyReferenceValueMap;
	#ifdef _TOOLBOX_TEST
		class TestListObject;
		class TestObject;
		class TestStackObject;
	#endif
	class UnsignedChar;
	class UnsignedInt;
	class UnsignedLongInt;
	class UnsignedShortInt;
	class Vertex;
	class VertexArray;
	class Void;
}
using namespace toolbox;


// ToolBox headers, level 0
#include <ToolBox\Exception.h>

// ToolBox headers, level 1
#include <ToolBox\ArrayList.h>
#include <ToolBox\HashMap.h>
	#include <ToolBox\HashSet.h>
#include <ToolBox\LinkedList.h>
#include <ToolBox\String.h>
#include <ToolBox\Stack.h>

// ToolBox headers, level 2
#include <ToolBox\Checksum.h>
#include <ToolBox\CommandLine.h>
#include <ToolBox\Config.h>
#include <ToolBox\Graph.h>
	#include <ToolBox\Directory.h>
#include <ToolBox\Logger.h>
#include <ToolBox\MD5Hasher.h>
#include <ToolBox\MP3FileInfos.h>
#include <ToolBox\Reference.h>
#include <ToolBox\Simulation.h>
#ifdef _TOOLBOX_TEST
	#include <ToolBox\TestObject.h>
#endif
#include <ToolBox\Wrappers.h>

// ToolBox inlines
#include <ToolBox\ArrayList.inl>
#include <ToolBox\Config.inl>
#include <ToolBox\Directory.inl>
#include <ToolBox\Exception.inl>
#include <ToolBox\Graph.inl>
#include <ToolBox\HashSet.inl>
#include <ToolBox\HashMap.inl>
#include <ToolBox\LinkedList.inl>
#include <ToolBox\Logger.inl>
#include <ToolBox\MD5Hasher.inl>
#include <ToolBox\MP3FileInfos.inl>
#include <ToolBox\Reference.inl>
#include <ToolBox\Simulation.inl>
#include <ToolBox\Stack.inl>
#include <ToolBox\String.inl>
#ifdef _TOOLBOX_TEST
	#include <ToolBox\TestObject.inl>
#endif
#include <ToolBox\Wrappers.inl>


#endif
