/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_MP3FILEINFOS_H
#define __TOOLBOX_MP3FILEINFOS_H


namespace toolbox
{
	/**
	 * An MP3FileInfos object stores all information about an MP3,
	 * except for the data itself.
	 * @author Thomas Jacob
	 */
	class MP3FileInfos
	{
		private:

			/**
			 * The artist name.
			 */
			char * Artist;

			/**
			 * The song name.
			 */
			char * Song;

			/**
			 * Capitalizes a given string.
			 * This method can be used to capitalize
			 * the artist and song names after they are
			 * retrieved through the getter methods.
			 * @param string The string to capitalize.
			 */
			static void Capitalize(char * string);

			/**
			 * Trims a given string.
			 * This method can be used to trim the
			 * artist and song names after they are
			 * retrieved through the getter methods.
			 * @param string The string to be trimmed.
			 */
			static void Trim(char * string);

		public:

			/**
			 * Creates a new MP3 file information.
			 */
			MP3FileInfos();

			/**
			 * Destroys the MP3 file information.
			 */
			~MP3FileInfos();

			/**
			 * Returns the artist name.
			 * @return The artist name.
			 */
			inline char * GetArtist();

			/**
			 * Returns the song name.
			 * @return The song name.
			 */
			inline char * GetSong();

			/**
			 * Capitalizes and trims the song and artist names.
			 */
			void Normalize();

			/**
			 * Sets the MP3 file information's attributes from a given file.
			 * @param path The path to the file.
			 */
			void ReadFrom(const char * path);

			/**
			 * Sets the artist name.
			 * @param artist The new artist name.
			 * The string will be copied by the method,
			 * so the memory is still yours.
			 */
			void SetArtist(char * artist);

			/**
			 * Sets the song name.
			 * @param song The new song name.
			 * The string will be copied by the method,
			 * so the memory is still yours.
			 */
			void SetSong(char * song);
	};
}


#endif
