/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


#ifndef __TOOLBOX_REFERENCE_INL
#define __TOOLBOX_REFERENCE_INL


// --- Class AutoDeleting ---

inline const ArrayList<Reference> & AutoDeleting::GetReferences() const
{
	ASSERTION_THIS;
	return References;
}


// --- Class Reference ---

inline void * Reference::GetReferrer() const
{
	ASSERTION_THIS;
	return Referrer;
}



// --- Class SingleReference ---

template <class ELEMENTTYPE> SingleReference<ELEMENTTYPE>::SingleReference(void * Referrer)
{
	ASSERTION_THIS;
	Element = NULL;
}

template <class ELEMENTTYPE> SingleReference<ELEMENTTYPE>::~SingleReference()
{
	ASSERTION_THIS;
	UnsetElement();
}

template <class ELEMENTTYPE> void SingleReference<ELEMENTTYPE>::AddElement(AutoDeleting * Element)
{
	ASSERTION_THIS;
	SetElement((ELEMENTTYPE *) Element);
}

template <class ELEMENTTYPE> inline ELEMENTTYPE * SingleReference<ELEMENTTYPE>::GetElement() const
{
	ASSERTION_THIS;
	return Element;
}

template <class ELEMENTTYPE> ArrayList<AutoDeleting> * SingleReference<ELEMENTTYPE>::GetElements() const
{
	ASSERTION_THIS;

	ArrayList<AutoDeleting> * Elements;
	if ((Elements = new ArrayList<AutoDeleting>) == NULL)
		throw EOutOfMemory::Get();

	if (Element != NULL)
		Elements->Append(Element);

	return Elements;
}

template <class ELEMENTTYPE> const AutoDeleting * SingleReference<ELEMENTTYPE>::ParameterConstraint = (ELEMENTTYPE *) 0;

template <class ELEMENTTYPE> void SingleReference<ELEMENTTYPE>::RemoveElement(AutoDeleting * Element)
{
	ASSERTION_THIS;
	if (this->Element == Element)
		UnsetElement();
}

template <class ELEMENTTYPE> void SingleReference<ELEMENTTYPE>::SetElement(ELEMENTTYPE * Element)
{
	ASSERTION_THIS;

	if (this->Element == Element)
		return;

	if (this->Element != NULL)
		this->Element->RemoveReference(this);

	this->Element = Element;

	if (Element != NULL)
		Element->AddReference(this);
}

template <class ELEMENTTYPE> inline void SingleReference<ELEMENTTYPE>::UnsetElement()
{
	ASSERTION_THIS;
	SetElement(NULL);
}


// --- Class ReferenceList ---

template <class ELEMENTTYPE> ReferenceList<ELEMENTTYPE>::ReferenceList()
{
	ASSERTION_THIS;
	if ((Elements = new ArrayList<ELEMENTTYPE>) == NULL)
		throw EOutOfMemory::Get();
}

template <class ELEMENTTYPE> ReferenceList<ELEMENTTYPE>::ReferenceList(long initSize)
{
	ASSERTION_THIS;
	if ((Elements = new ArrayList<ELEMENTTYPE>(initSize)) == NULL)
		throw EOutOfMemory::Get();
}

template <class ELEMENTTYPE> ReferenceList<ELEMENTTYPE>::ReferenceList(long initSize,
	int minimumFillRatio, int desiredFillRatio)
{
	ASSERTION_THIS;
	if ((Elements = new ArrayList<ELEMENTTYPE>(initSize, minimumFillRatio, desiredFillRatio)) == NULL)
		throw EOutOfMemory::Get();
}

template <class ELEMENTTYPE> ReferenceList<ELEMENTTYPE>::~ReferenceList()
{
	ASSERTION_THIS;

	if (Elements != NULL)
	{
		RemoveAll();
		delete Elements;
		Elements = NULL;
	}
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::AddElement(AutoDeleting * Element)
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	Elements->Append(Element);
	Element->AddReference(this);
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::Append(ELEMENTTYPE * Element)
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	Elements->Append(Element);
	Element->AddReference(this);
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::AppendAll(const ArrayList<ELEMENTTYPE> * Elements)
{
	ASSERTION_THIS;
	ASSERTION(Elements != NULL);

	this->Elements->AppendAll(Elements);
	for (int i=0; i<Elements->GetCount(); i++)
		Elements->Get(i)->AddReference(this);
}

template <class ELEMENTTYPE> bool ReferenceList<ELEMENTTYPE>::Contains(const ELEMENTTYPE * Element) const
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	return Elements->Contains(Element, ArrayList::Compare);
}

template <class ELEMENTTYPE> bool ReferenceList<ELEMENTTYPE>::Contains(const ELEMENTTYPE * Element,
	int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	return Elements->Contains(Element, compare);
}

template <class ELEMENTTYPE> int ReferenceList<ELEMENTTYPE>::Find(const ELEMENTTYPE * Element) const
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	return Elements->Find(Element, ArrayList::Compare);
}

template <class ELEMENTTYPE> int ReferenceList<ELEMENTTYPE>::Find(const ELEMENTTYPE * Element,
	int (* compare)(const ELEMENTTYPE * Element1, const ELEMENTTYPE * Element2)) const
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	return Elements->Find(Element, compare);
}

template <class ELEMENTTYPE> ELEMENTTYPE * ReferenceList<ELEMENTTYPE>::Get(long nr) const
{
	ASSERTION_THIS;
	return Elements->Get(nr);
}

template <class ELEMENTTYPE> inline long ReferenceList<ELEMENTTYPE>::GetCount() const
{
	ASSERTION_THIS;
	return Elements->GetCount();
}

template <class ELEMENTTYPE> inline int ReferenceList<ELEMENTTYPE>::GetDesiredFillRatio() const
{
	ASSERTION_THIS;
	return Elements->GetDesiredFillRatio();
}

template <class ELEMENTTYPE> ArrayList<AutoDeleting> * ReferenceList<ELEMENTTYPE>::GetElements() const
{
	ASSERTION_THIS;
	return Elements->GetElements();
}

template <class ELEMENTTYPE> inline int ReferenceList<ELEMENTTYPE>::GetFillRatio() const
{
	ASSERTION_THIS;
	return Elements->GetFillRatio();
}

template <class ELEMENTTYPE> ELEMENTTYPE * ReferenceList<ELEMENTTYPE>::GetFirst() const
{
	ASSERTION_THIS;
	return Elements->GetFirst();
}

template <class ELEMENTTYPE> ELEMENTTYPE * ReferenceList<ELEMENTTYPE>::GetLast() const
{
	ASSERTION_THIS;
	return Elements->GetLast();
}

template <class ELEMENTTYPE> inline int ReferenceList<ELEMENTTYPE>::GetMinimumFillRatio() const
{
	ASSERTION_THIS;
	return Elements->GetMinimumFillRatio();
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::Insert(const ELEMENTTYPE * Element, long nr)
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	Elements->Insert(Element, nr);
	Element->AddReference(this);
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::InsertAll(const ArrayList<ELEMENTTYPE> * Elements, long nr)
{
	ASSERTION_THIS;
	ASSERTION(Elements != NULL);

	this->Elements->InsertAll(Elements, nr);
	for (int i=0; i<Elements->GetCount(); i++)
		Elements->Get(i)->AddReference(this);
}

template <class ELEMENTTYPE> inline bool ReferenceList<ELEMENTTYPE>::IsEmpty() const
{
	ASSERTION_THIS;
	return Elements->IsEmpty();
}

template <class ELEMENTTYPE> const AutoDeleting * ReferenceList<ELEMENTTYPE>::ParameterConstraint = (ELEMENTTYPE *) 0;

template <class ELEMENTTYPE> inline void ReferenceList<ELEMENTTYPE>::Prepend(const ELEMENTTYPE * Element)
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	Elements->Prepend(Element);
	Element->AddReference(this);
}

template <class ELEMENTTYPE> inline void ReferenceList<ELEMENTTYPE>::PrependAll(const ArrayList<ELEMENTTYPE> * Elements)
{
	ASSERTION_THIS;
	ASSERTION(Elements != NULL);

	this->Elements->PrependAll(Elements);
	for (int i=0; i<Elements->GetCount(); i++)
		Elements->Get(i)->AddReference(this);
}

template <class ELEMENTTYPE> ELEMENTTYPE * ReferenceList<ELEMENTTYPE>::Replace(ELEMENTTYPE * NewElement, long nr)
{
	ASSERTION_THIS;
	ASSERTION(NewElement != NULL);
}

template <class ELEMENTTYPE> bool ReferenceList<ELEMENTTYPE>::Remove(long nr)
{
	ASSERTION_THIS;

	if (Elements->Unlink(nr) != NULL)
		Element->RemoveReference(this);
}

template <class ELEMENTTYPE> bool ReferenceList<ELEMENTTYPE>::Remove(long from, long count)
{
	ASSERTION_THIS;

	for (int i = from; i < from + count; i++)
		if (i >= 0 && i<Elements->GetCount())
			Elements->Get(i)->RemoveReference(this);

	Elements->Unlink(nr, count);
}

template <class ELEMENTTYPE> bool ReferenceList<ELEMENTTYPE>::Remove(ELEMENTTYPE * Element)
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	if (Elements->Unlink(Element) != NULL)
		Element->RemoveReference(this);
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::RemoveAll()
{
	ASSERTION_THIS;

	for (int i=0; i<Elements->GetCount(); i++)
		Elements->Get(i)->RemoveReference(this);

	this->Elements->UnlinkAll();
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::RemoveElement(AutoDeleting * Element)
{
	ASSERTION_THIS;
	ASSERTION(Element != NULL);

	if (Elements->Unlink(Element) != NULL)
		Element->RemoveReference(this);
}

template <class ELEMENTTYPE> inline bool ReferenceList<ELEMENTTYPE>::RemoveFirst()
{
	ASSERTION_THIS;

	if (Elements->UnlinkFirst(Element) != NULL)
		Element->RemoveReference(this);
}

template <class ELEMENTTYPE> bool ReferenceList<ELEMENTTYPE>::RemoveLast()
{
	ASSERTION_THIS;

	if (Elements->UnlinkLast(Element) != NULL)
		Element->RemoveReference(this);
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::Shuffle()
{
	ASSERTION_THIS;
	Elements->Shuffle();
}

template <class ELEMENTTYPE> inline void ReferenceList<ELEMENTTYPE>::Sort()
{
	ASSERTION_THIS;
	Elements->Sort(ArrayList::Compare);
}

template <class ELEMENTTYPE> void ReferenceList<ELEMENTTYPE>::Sort(int (* compare)(const ELEMENTTYPE * Element1,
	const ELEMENTTYPE * Element2))
{
	ASSERTION_THIS;
	ASSERTION(compare != NULL);
	Elements->Sort(compare);
}


#endif
