/*
 * This file is part of the ToolBox.
 * Copyright Thomas Jacob.
 *
 * READ README.TXT BEFORE USE!!
 */


// Main header file
#include <ToolBox\Headers.h>


// --- Class String ---

String::String()
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Size = _TB_ST_SIZEINCREMENT;
	if ((Data = new char[Size]) == NULL)
		throw EOutOfMemory::Get();
	Data[0] = 0;
}

String::String(const class String & String)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Size = String.Size;
	if ((Data = new char[Size]) == NULL)
		throw EOutOfMemory::Get();
	memcpy(Data, String.Data, Size);
}

String::String(const class String * String)
{
	ASSERTION_THIS;
	ASSERTION(String != NULL);

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Size = String->Size;
	if ((Data = new char[Size]) == NULL)
		throw EOutOfMemory::Get();
	memcpy(Data, String->Data, Size);
}

String::String(const char * string)
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	unsigned int minimumSize = strlen(string) + 1;
	Allocate(minimumSize);
	memcpy(Data, string, minimumSize);
}

String::String(const String & String, unsigned int fromIndex, unsigned int length)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	unsigned int currentLength = strlen(String.Data);
	if (fromIndex > currentLength)
		fromIndex = currentLength;
	if (length > length - fromIndex)
		length = length - fromIndex;

	unsigned int minimumSize = length + 1;
	Allocate(minimumSize);

	memcpy(Data, &String.Data[fromIndex], length);
	Data[length] = 0;
}

String::String(const String * String, unsigned int fromIndex, unsigned int length)
{
	ASSERTION_THIS;
	ASSERTION(String != NULL);

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	unsigned int currentLength = strlen(String->Data);
	if (fromIndex > currentLength)
		fromIndex = currentLength;
	if (length > length - fromIndex)
		length = length - fromIndex;

	unsigned int minimumSize = length + 1;
	Allocate(minimumSize);

	memcpy(Data, &String->Data[fromIndex], length);
	Data[length] = 0;
}

String::String(const char * string, unsigned int fromIndex, unsigned int length)
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	unsigned int minimumSize = length + 1;
	Allocate(minimumSize);

	memcpy(Data, &string[fromIndex], length);
	Data[length] = 0;
}

String::String(char character)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Size = _TB_ST_SIZEINCREMENT;
	if ((Data = new char[Size]) == NULL)
		throw EOutOfMemory::Get();
	Data[0] = character;
	Data[1] = 0;
}

String::String(int number)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Allocate(12);
	_itoa(number, Data, 10);
}

String::String(long number)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Allocate(12);
	_ltoa(number, Data, 10);
}

String::String(unsigned long number)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Allocate(12);
	_ultoa(number, Data, 10);
}

String::String(bool boolean)
{
	ASSERTION_THIS;

	#ifdef _TOOLBOX_TEST
		InstanceCount++;
	#endif

	Allocate(6);

	if (boolean)
		memcpy(Data, "true", 5);
	else
		memcpy(Data, "false", 6);
}

String::~String()
{
	ASSERTION_THIS;

	if (Data != NULL)
		delete Data;

	#ifdef _TOOLBOX_TEST
		InstanceCount--;
	#endif
}

void String::operator =(const char * string)
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length = strlen(string);
	unsigned int minimumSize = length + 1;

	if (Size < minimumSize)
	{
		delete Data;
		Allocate(minimumSize);
	}

	memcpy(Data, string, length);
	Data[length] = 0;
}

void String::operator =(char character)
{
	ASSERTION_THIS;

	if (Size < 2)
	{
		delete Data;
		Allocate(2);
	}

	Data[0] = character;
	Data[1] = 0;
}

void String::operator =(int number)
{
	ASSERTION_THIS;

	if (Size < 12)
	{
		delete Data;
		Allocate(12);
	}

	_itoa(number, Data, 10);
}

void String::operator =(long number)
{
	ASSERTION_THIS;

	if (Size < 12)
	{
		delete Data;
		Allocate(12);
	}

	_ltoa(number, Data, 10);
}

void String::operator =(unsigned long number)
{
	ASSERTION_THIS;

	if (Size < 12)
	{
		delete Data;
		Allocate(12);
	}

	_ultoa(number, Data, 10);
}

void String::operator =(bool boolean)
{
	ASSERTION_THIS;

	if (Size < 6)
	{
		delete Data;
		Allocate(6);
	}

	if (boolean)
		memcpy(Data, "true", 5);
	else
		memcpy(Data, "false", 6);
}

void String::Allocate(unsigned int minimumSize)
{
	ASSERTION_THIS;

	Size = (minimumSize + _TB_ST_SIZEINCREMENT)
		/ _TB_ST_SIZEINCREMENT * _TB_ST_SIZEINCREMENT;
	if ((Data = new char[Size]) == NULL)
		throw EOutOfMemory::Get();
}

void String::Append(const char * string)
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length1 = strlen(Data);
	unsigned int length2 = strlen(string);
	unsigned int minimumSize = length1 + length2 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length1);
		delete oldData;
	}

	memcpy(&Data[length1], string, length2);
	Data[length1 + length2] = 0;
}

void String::Append(char character)
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);
	unsigned int minimumSize = length + 1 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length);
		delete oldData;
	}

	Data[length] = character;
	Data[length + 1] = 0;
}

void String::Append(int number)
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);
	unsigned int minimumSize = length + 11 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length);
		delete oldData;
	}

	_itoa(number, &Data[length], 10);
}

void String::Append(long number)
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);
	unsigned int minimumSize = length + 11 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length);
		delete oldData;
	}

	_ltoa(number, &Data[length], 10);
}

void String::Append(unsigned long number)
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);
	unsigned int minimumSize = length + 11 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length);
		delete oldData;
	}

	_ultoa(number, &Data[length], 10);
}

void String::Append(bool boolean)
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);
	unsigned int minimumSize = length + (boolean ? 4 : 5) + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length);
		delete oldData;
	}

	if (boolean)
		memcpy(&Data[length], "true", 5);
	else
		memcpy(&Data[length], "false", 6);
}

int String::Compare(const char * string1, const char * string2)
{
	ASSERTION(string1 != NULL);
	ASSERTION(string2 != NULL);

	return strcmp(string1, string2);
}

int String::CompareIC(const char * string1, const char * string2)
{
	ASSERTION(string1 != NULL);
	ASSERTION(string2 != NULL);

	return stricmp(string1, string2);
}

void String::Cut(unsigned int fromIndex)
{
	ASSERTION_THIS;

	unsigned int currentLength = strlen(Data);
	if (fromIndex > currentLength)
		return;

	Data[fromIndex] = 0;
}

void String::Cut(unsigned int fromIndex, unsigned int length)
{
	ASSERTION_THIS;

	unsigned int currentLength = strlen(Data);
	if (fromIndex > currentLength)
		return;
	if (length > currentLength - fromIndex)
		length = currentLength - fromIndex;

	memmove(&Data[fromIndex], &Data[fromIndex + length],
		currentLength - fromIndex - length + 1);
}

bool String::EndsWith(const char * string) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length1 = strlen(Data);
	unsigned int length2 = strlen(string);
	if (length2 > length1)
		return false;

	return strncmp(&Data[length1 - length2], string, length2) == 0;
}

bool String::EndsWithIC(const char * string) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length1 = strlen(Data);
	unsigned int length2 = strlen(string);
	if (length2 > length1)
		return false;

	return strnicmp(&Data[length1 - length2], string, length2) == 0;
}

int String::Find(const char * string) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	char * occurrence = strstr(Data, string);
	return occurrence != NULL ? occurrence - Data : -1;
}

int String::Find(const char * string, unsigned int fromIndex) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	if (fromIndex > strlen(Data))
		return -1;

	char * occurrence = strstr(&Data[fromIndex], string);
	return occurrence != NULL ? occurrence - Data : -1;
}

int String::FindIC(const char * string, unsigned int fromIndex) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int dataLength = strlen(Data);
	unsigned int stringLength = strlen(string);

	if (stringLength > dataLength)
		return -1;

	for (unsigned int i = fromIndex; i < dataLength - stringLength; i++)
		if (strnicmp(&Data[i], string, stringLength) == 0)
			return i;

	return -1;
}

int String::FindLast(const char * string, unsigned int fromIndex) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int dataLength = strlen(Data);
	unsigned int stringLength = strlen(string);

	if (stringLength > dataLength)
		return -1;

	if (fromIndex > dataLength - stringLength)
		fromIndex = dataLength - stringLength;

	for (int i = fromIndex; i >= 0; i--)
		if (strncmp(&Data[i], string, stringLength) == 0)
			return i;

	return -1;
}

int String::FindLastIC(const char * string, unsigned int fromIndex) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int dataLength = strlen(Data);
	unsigned int stringLength = strlen(string);

	if (stringLength > dataLength)
		return -1;

	if (fromIndex > dataLength - stringLength)
		fromIndex = dataLength - stringLength;

	for (int i = fromIndex; i >= 0; i--)
		if (strnicmp(&Data[i], string, stringLength) == 0)
			return i;

	return -1;
}

void String::Format(const char * format, ...)
{
	ASSERTION_THIS;
	ASSERTION(format != NULL);

	va_list arguments;
	va_start(arguments, format);

	FormatFromVargs(format, arguments);

	va_end(arguments);
}

void String::FormatFromVargs(const char * format, const va_list & arguments)
{
	ASSERTION_THIS;
	ASSERTION(format != NULL);

	unsigned int written = _vsnprintf(Data, Size - 1, format, arguments);
	while ((written < 0 || written >= Size - 1) && Size < 0x20000000L)
	{
		delete Data;
		Allocate(Size * 2);
		written = _vsnprintf(Data, Size - 1, format, arguments);
	}

	if (Size >= 0x20000000L)
	{
		delete Data;
		Allocate(_TB_ST_SIZEINCREMENT);
		Data[0] = 0;

		throw NEW_EILLEGALARGUMENT("Formatted string requires too much space (>512MB)");
	}
}

String * String::HeapClone() const
{
	ASSERTION_THIS;

	String * clone;
	if ((clone = new String(this)) == NULL)
		throw EOutOfMemory::Get();

	return clone;
}

void String::Insert(const char * string, unsigned int index)
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length1 = strlen(Data);
	unsigned int length2 = strlen(string);
	unsigned int minimumSize = length1 + length2 + 1;

	if (index > length1)
		index = length1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length1);
		delete oldData;
	}

	memmove(&Data[index + length2], &Data[index], length1 - index);
	memcpy(&Data[index], string, length2);
	Data[length1 + length2] = 0;
}

void String::Insert(char character, unsigned int index)
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);
	unsigned int minimumSize = length + 1 + 1;

	if (index > length)
		index = length;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length);
		delete oldData;
	}

	memmove(&Data[index + 1], &Data[index], length - index);
	Data[index] = character;
	Data[length + 1] = 0;
}

void String::Insert(int number, unsigned int index)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int minimumSize = length1 + 11 + 1;

	if (index > length1)
		index = length1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length1);
		delete oldData;
	}

	memmove(&Data[index + 11], &Data[index], length1 - index);
	_itoa(number, &Data[index], 10);
	unsigned int length2 = strlen(&Data[index]);
	memmove(&Data[index + length2], &Data[index + 11], length1 - index);
	Data[length1 + length2] = 0;
}

void String::Insert(long number, unsigned int index)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int minimumSize = length1 + 11 + 1;

	if (index > length1)
		index = length1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length1);
		delete oldData;
	}

	memmove(&Data[index + 11], &Data[index], length1 - index);
	_ltoa(number, &Data[index], 10);
	unsigned int length2 = strlen(&Data[index]);
	memmove(&Data[index + length2], &Data[index + 11], length1 - index);
	Data[length1 + length2] = 0;
}

void String::Insert(unsigned long number, unsigned int index)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int minimumSize = length1 + 11 + 1;

	if (index > length1)
		index = length1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length1);
		delete oldData;
	}

	memmove(&Data[index + 11], &Data[index], length1 - index);
	_ultoa(number, &Data[index], 10);
	unsigned int length2 = strlen(&Data[index]);
	memmove(&Data[index + length2], &Data[index + 11], length1 - index);
	Data[length1 + length2] = 0;
}

void String::Insert(bool boolean, unsigned int index)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int length2 = boolean ? 4 : 5;
	unsigned int minimumSize = length1 + length2 + 1;

	if (index > length1)
		index = length1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(Data, oldData, length1);
		delete oldData;
	}

	memmove(&Data[index + length2], &Data[index], length1 - index);
	memcpy(&Data[index], boolean ? "true" : "false", length2);
	Data[length1 + length2] = 0;
}

#ifdef _TOOLBOX_TEST
	int String::InstanceCount = 0;
#endif

bool String::IsCharacter(unsigned char character)
{
	return IsUpperCharacter(character) || IsLowerCharacter(character);
}

bool String::IsDigit(unsigned char character)
{
	return character >= '0' && character <= '9';
}

bool String::IsLowerCharacter(unsigned char character)
{
	return character >= 'a' && character <= 'z'
		|| character == 170 || character == 181 || character == 186
		|| character >= 223 && character != 247;
}

bool String::IsUpperCharacter(unsigned char character)
{
	return character >= 'A' && character <= 'Z'
		|| character >= 192 && character <= 222 && character != 215;
}

bool String::IsValid(unsigned char character)
{
	return character <= 126 || character >= 160;
}

bool String::IsWhiteSpace(unsigned char character)
{
	return character >= 9 && character <= 13
		|| character >= 28 && character <= 32 || character == 160;
}

void String::Prepend(const char * string)
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length1 = strlen(Data);
	unsigned int length2 = strlen(string);
	unsigned int minimumSize = length2 + length1 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(&Data[length2], oldData, length1);
		delete oldData;
	}
	else
		memmove(&Data[length2], Data, length1);

	memcpy(Data, string, length2);
	Data[length2 + length1] = 0;
}

void String::Prepend(char character)
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);
	unsigned int minimumSize = length + 1 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(&Data[1], oldData, length);
		delete oldData;
	}
	else
		memcpy(&Data[1], Data, length);

	Data[0] = character;
	Data[length + 1] = 0;
}

void String::Prepend(int number)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int minimumSize = length1 + 11 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(&Data[11], oldData, length1);
		delete oldData;
	}
	else
		memcpy(&Data[11], Data, length1);

	_itoa(number, Data, 10);
	unsigned int length2 = strlen(Data);
	memmove(&Data[length2], &Data[11], length1);
	Data[length1 + length2] = 0;
}

void String::Prepend(long number)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int minimumSize = length1 + 11 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(&Data[11], oldData, length1);
		delete oldData;
	}
	else
		memcpy(&Data[11], Data, length1);

	_ltoa(number, Data, 10);
	unsigned int length2 = strlen(Data);
	memmove(&Data[length2], &Data[11], length1);
	Data[length1 + length2] = 0;
}

void String::Prepend(unsigned long number)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int minimumSize = length1 + 11 + 1;

	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(&Data[11], oldData, length1);
		delete oldData;
	}
	else
		memcpy(&Data[11], Data, length1);

	_ultoa(number, Data, 10);
	unsigned int length2 = strlen(Data);
	memmove(&Data[length2], &Data[11], length1);
	Data[length1 + length2] = 0;
}

void String::Prepend(bool boolean)
{
	ASSERTION_THIS;

	unsigned int length1 = strlen(Data);
	unsigned int length2 = boolean ? 4 : 5;

	unsigned int minimumSize = length2 + length1 + 1;
	if (Size < minimumSize)
	{
		char * oldData = Data;
		Allocate(minimumSize);

		memcpy(&Data[length2], oldData, length1);
		delete oldData;
	}
	else
		memmove(&Data[length2], Data, length1);

	memcpy(Data, boolean ? "true" : "false", length2);
	Data[length2 + length1] = 0;
}

bool String::ReadFrom(const char * fileName, bool textMode)
{
	ASSERTION_THIS;
	ASSERTION(fileName != NULL);
	ASSERTION(fileName[0] != 0);

	const char * mode = textMode ? "rt" : "rb";

	FILE * file;
	if ((file = fopen(fileName, mode)) == NULL)
	{
		Clear();
		return false;
	}

	bool success = ReadFrom(file);
	fclose(file);

	return success;
}

bool String::ReadFrom(FILE * file)
{
	ASSERTION_THIS;
	ASSERTION(file != NULL);

	Clear();

	char buffer[1025];
	int read = fread(buffer, 1, 1024, file);
	while (read > 0)
	{
		buffer[read] = 0;
		Append(buffer);
		read = fread(buffer, 1, 1024, file);
	}

	if (ferror(file))
	{
		Clear();
		return false;
	}
	else
		return true;
}

void String::Replace(const char * text, const char * replacement)
{
	ASSERTION_THIS;
	ASSERTION(text != NULL);
	ASSERTION(replacement != NULL);

	unsigned int textLength = strlen(text);
	unsigned int replacementLength = strlen(replacement);

	int occurrence = Find(text);
	while (occurrence >= 0)
	{
		*this = Substring(0, occurrence) + replacement + Substring(occurrence + textLength);
		occurrence = Find(text, occurrence + replacementLength);
	}
}

void String::Replace(char character, char replacement)
{
	ASSERTION_THIS;

	char * occurrence = strchr(Data, character);
	while (occurrence != NULL)
	{
		occurrence[0] = replacement;
		occurrence = strchr(&occurrence[1], character);
	}
}

#ifdef _TOOLBOX_TEST
	void String::RunTestSuite(int * performedTests, int * failedTests)
	{
		{
			String * TestString = new String;
			if (String::InstanceCount != 1) {printf("Test String[1] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString->Size != _TB_ST_SIZEINCREMENT) {printf("Test String[2] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString->Data, "") != 0) {printf("Test String[3] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			delete TestString;

			String * TestString2 = new String("Test");
			if (String::InstanceCount != 1) {printf("Test String[4] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString2->Size != _TB_ST_SIZEINCREMENT) {printf("Test String[5] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString2->Data, "Test") != 0) {printf("Test String[6] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString = new String(TestString2);
			if (String::InstanceCount != 2) {printf("Test String[7] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString->Size != _TB_ST_SIZEINCREMENT) {printf("Test String[8] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString->Data, "Test") != 0) {printf("Test String[9] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			delete TestString;
			delete TestString2;
			if (String::InstanceCount != 0) {printf("Test String[10] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString = new String('x');
			if (String::InstanceCount != 1) {printf("Test String[11] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString->Size != _TB_ST_SIZEINCREMENT) {printf("Test String[12] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString->Data, "x") != 0) {printf("Test String[13] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			delete TestString;
			if (String::InstanceCount != 0) {printf("Test String[14] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString = new String(12345678UL);
			if (String::InstanceCount != 1) {printf("Test String[15] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString->Size != _TB_ST_SIZEINCREMENT) {printf("Test String[16] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString->Data, "12345678") != 0) {printf("Test String[17] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			delete TestString;
			if (String::InstanceCount != 0) {printf("Test String[18] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString = new String(87654321L);
			if (String::InstanceCount != 1) {printf("Test String[19] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString->Size != _TB_ST_SIZEINCREMENT) {printf("Test String[20] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString->Data, "87654321") != 0) {printf("Test String[21] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			delete TestString;
			if (String::InstanceCount != 0) {printf("Test String[22] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString = new String(false);
			if (String::InstanceCount != 1) {printf("Test String[23] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString->Size != _TB_ST_SIZEINCREMENT) {printf("Test String[24] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString->Data, "false") != 0) {printf("Test String[25] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			delete TestString;
			if (String::InstanceCount != 0) {printf("Test String[26] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String TestString3;
			TestString3 += "Test";
			if (TestString3.Size != _TB_ST_SIZEINCREMENT) {printf("Test String[27] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString3.Data, "Test") != 0) {printf("Test String[28] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 += "Some more text!!!";
			if (TestString3.Size != _TB_ST_SIZEINCREMENT * 2) {printf("Test String[29] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString3.Data, "TestSome more text!!!") != 0) {printf("Test String[30] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 += 12345678UL;
			if (strcmp(TestString3.Data, "TestSome more text!!!12345678") != 0) {printf("Test String[31] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 += 87654321L;
			if (strcmp(TestString3.Data, "TestSome more text!!!1234567887654321") != 0) {printf("Test String[32] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 += true;
			TestString3 += false;
			if (TestString3.Size <= _TB_ST_SIZEINCREMENT * 2) {printf("Test String[33] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (strcmp(TestString3.Data, "TestSome more text!!!1234567887654321truefalse") != 0) {printf("Test String[34] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String TestString4 = String("") + "Test" + String("Some more text") + 34UL + (-34L) + true + 'x';
			if (strcmp(TestString4.Data, "TestSome more text34-34truex") != 0) {printf("Test String[35] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 = "Another test";
			if (strcmp(TestString3.Data, "Another test") != 0) {printf("Test String[36] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			String Clone = TestString3.Clone();
			if (strcmp(Clone.Data, "Another test") != 0) {printf("Test String[37] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String First("First");
			String Second("Second");
			if (First > First) {printf("Test String[38] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First > Second) {printf("Test String[39] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!(First >= First)) {printf("Test String[40] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First >= Second) {printf("Test String[41] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First < First) {printf("Test String[42] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!(First < Second)) {printf("Test String[43] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!(First <= First)) {printf("Test String[44] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!(First <= Second)) {printf("Test String[45] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!(First == First)) {printf("Test String[46] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First == Second) {printf("Test String[47] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First != First) {printf("Test String[48] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!(First != Second)) {printf("Test String[49] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			if (First[2] != 'r') {printf("Test String[50] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First[5] != 0) {printf("Test String[51] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (((const char *) First)[2] != 'r') {printf("Test String[52] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First.GetLength() != 5) {printf("Test String[53] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String Template("This is a text, is a template text");
			if (Template.Contains("is a Text")) {printf("Test String[54] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.Contains("is a text")) {printf("Test String[55] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.Contains("This is a text, is a template text")) {printf("Test String[56] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.StartsWith("This is A ")) {printf("Test String[57] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.StartsWith("This is a ")) {printf("Test String[58] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.StartsWith("This is a text, is a template text")) {printf("Test String[59] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.StartsWith("")) {printf("Test String[60] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.EndsWith("Ate text")) {printf("Test String[61] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.EndsWith("ate text")) {printf("Test String[62] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.EndsWith("This is a text, is a template text")) {printf("Test String[63] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Template.EndsWith("")) {printf("Test String[64] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.Find("is a") != 5) {printf("Test String[65] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.Find("is a", 5) != 5) {printf("Test String[66] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.Find("is a", 6) != 16) {printf("Test String[67] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.Substring(10, 4) != "text") {printf("Test String[68] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.Substring(30, 4) != "text") {printf("Test String[69] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Template.Substring(30) != "text") {printf("Test String[70] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			Template.Replace(" text", " test");
			if (Template != "This is a test, is a template test") {printf("Test String[71] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			Template.Replace('i', 'I');
			if (Template != "ThIs Is a test, Is a template test") {printf("Test String[72] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			Template.ToLowerCase();
			if (Template != "this is a test, is a template test") {printf("Test String[73] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			Template.ToUpperCase();
			if (Template != "THIS IS A TEST, IS A TEMPLATE TEST") {printf("Test String[74] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String Formatted;
			Formatted.Format("%d words %d say: %s %s%c", 2, 2, "it", "works", '!');
			if (Formatted.Data == "2 words 2 say: it works!") {printf("Test String[75] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!(Formatted.Data != "2 words 2 say: it works!")) {printf("Test String[76] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String UntrimmedLeft("   some  untrimmed \n text \n  \r");
			UntrimmedLeft.TrimLeft();
			if (UntrimmedLeft != "some  untrimmed \n text \n  \r") {printf("Test String[77] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String UntrimmedRight("   some  untrimmed \n text \n  \r");
			UntrimmedRight.TrimRight();
			if (UntrimmedRight != "   some  untrimmed \n text") {printf("Test String[78] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String Untrimmed("   some  untrimmed \n text \n  \r");
			Untrimmed.Trim();
			if (Untrimmed != "some  untrimmed \n text") {printf("Test String[79] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String Untrimmed2("        ");
			Untrimmed2.Trim();
			if (Untrimmed2 != "") {printf("Test String[80] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!Untrimmed2.IsEmpty()) {printf("Test String[81] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String Untrimmed3("xxxxxxxx");
			Untrimmed3.Trim();
			if (Untrimmed3 != "xxxxxxxx") {printf("Test String[82] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (Untrimmed3.IsEmpty()) {printf("Test String[83] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String Untrimmed4("");
			Untrimmed4.Trim();
			if (Untrimmed4 != "") {printf("Test String[84] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String Untrimmed3Clone = Untrimmed3.Clone();
			if (Untrimmed3Clone != Untrimmed3) {printf("Test String[85] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (&Untrimmed3Clone == &Untrimmed3) {printf("Test String[86] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 = "A little append, prepend, insert test";
			TestString3.Append("hallo");
			if (strcmp(TestString3.Data, "A little append, prepend, insert testhallo") != 0)
				{printf("Test String[87] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Append(1);
			if (strcmp(TestString3.Data, "A little append, prepend, insert testhallo1") != 0)
				{printf("Test String[88] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Append(false);
			if (strcmp(TestString3.Data, "A little append, prepend, insert testhallo1false") != 0)
				{printf("Test String[89] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Append(true);
			if (strcmp(TestString3.Data, "A little append, prepend, insert testhallo1falsetrue") != 0)
				{printf("Test String[90] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Append('x');
			if (strcmp(TestString3.Data, "A little append, prepend, insert testhallo1falsetruex") != 0)
				{printf("Test String[91] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 = "A little append, prepend, insert test";
			TestString3.Prepend("hallo");
			if (strcmp(TestString3.Data, "halloA little append, prepend, insert test") != 0)
				{printf("Test String[92] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Prepend(1);
			if (strcmp(TestString3.Data, "1halloA little append, prepend, insert test") != 0)
				{printf("Test String[93] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Prepend(false);
			if (strcmp(TestString3.Data, "false1halloA little append, prepend, insert test") != 0)
				{printf("Test String[94] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Prepend(true);
			if (strcmp(TestString3.Data, "truefalse1halloA little append, prepend, insert test") != 0)
				{printf("Test String[95] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Prepend('x');
			if (strcmp(TestString3.Data, "xtruefalse1halloA little append, prepend, insert test") != 0)
				{printf("Test String[96] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 = "A little append, prepend, insert test";
			TestString3.Insert("hallo", 3);
			if (strcmp(TestString3.Data, "A lhalloittle append, prepend, insert test") != 0)
				{printf("Test String[97] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Insert(1, 13);
			if (strcmp(TestString3.Data, "A lhalloittle1 append, prepend, insert test") != 0)
				{printf("Test String[98] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Insert(false, 23);
			if (strcmp(TestString3.Data, "A lhalloittle1 append, falseprepend, insert test") != 0)
				{printf("Test String[99] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Insert(true, strlen(TestString3));
			if (strcmp(TestString3.Data, "A lhalloittle1 append, falseprepend, insert testtrue") != 0)
				{printf("Test String[100] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			TestString3.Insert('x', 0);
			if (strcmp(TestString3.Data, "xA lhalloittle1 append, falseprepend, insert testtrue") != 0)
				{printf("Test String[101] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			TestString3 = "This is some text";
			if (!TestString3.StartsWithIC("this"))
				{printf("Test String[102] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.StartsWithIC("That"))
				{printf("Test String[103] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (!TestString3.EndsWithIC("Text"))
				{printf("Test String[104] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.EndsWithIC("test"))
				{printf("Test String[105] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			if (!First.Equals(First)) {printf("Test String[106] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (First.Equals(Second)) {printf("Test String[107] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			String FirstLower(First);
			FirstLower.ToLowerCase();
			if (!(FirstLower.EqualsIC(First))) {printf("Test String[108] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (FirstLower.EqualsIC(Second)) {printf("Test String[109] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			if (TestString3.Find("is", 50) != -1) {printf("Test String[110] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			if (TestString3.FindLast("is") != 5) {printf("Test String[111] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.FindLast("is", 4) != 2) {printf("Test String[112] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.FindLast("is", 5) != 5) {printf("Test String[113] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.FindLast("is", 50) != 5) {printf("Test String[114] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.FindLast("ist") != -1) {printf("Test String[115] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.FindLast("ist", 5) != -1) {printf("Test String[116] failed!\n"); (*failedTests)++;}
			(*performedTests)++;
			if (TestString3.FindLast("is", 1) != -1) {printf("Test String[117] failed!\n"); (*failedTests)++;}
			(*performedTests)++;

			bool failed = false;
			for (int i=0; i<256; i++)
			{
				if ((IsCharacter(i) ? 'T' : 'F') !=
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FTTTTTTTTTTTTTTTTTTTTTTTTTTFFFFFFTTTTTTTTTTTTTTTTTTTTTTTTTTFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFTFFFFFFFFFFTFFFFTFFFFF"
					"TTTTTTTTTTTTTTTTTTTTTTTFTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTFTTTTTTTT"
					[i])
				{
					printf("Test String[118] failed: character test of character %d!\n", i);
					failed = true;
				}

				if ((IsDigit(i) ? 'T' : 'F') !=
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFTTTTTTTTTTFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					[i])
				{
					printf("Test String[118] failed: digit test of character %d!\n", i);
					failed = true;
				}

				if ((IsLowerCharacter(i) ? 'T' : 'F') !=
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFTTTTTTTTTTTTTTTTTTTTTTTTTTFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFTFFFFFFFFFFTFFFFTFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFTTTTTTTTTTTTTTTTTTTTTTTTFTTTTTTTT"
					[i])
				{
					printf("Test String[118] failed: lower character test of character %d!\n", i);
					failed = true;
				}

				if ((IsUpperCharacter(i) ? 'T' : 'F') !=
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FTTTTTTTTTTTTTTTTTTTTTTTTTTFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"TTTTTTTTTTTTTTTTTTTTTTTFTTTTTTTFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					[i])
				{
					printf("Test String[118] failed: upper character test of character %d!\n", i);
					failed = true;
				}

				if ((IsValid(i) ? 'T' : 'F') !=
					"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT"
					"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT"
					"TTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTTT"
					[i])
				{
					printf("Test String[118] failed: validity test of character %d!\n", i);
					failed = true;
				}

				if ((IsWhiteSpace(i) ? 'T' : 'F') !=
					"FFFFFFFFFTTTTTFFFFFFFFFFFFFFTTTTTFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFTFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					"FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF"
					[i])
				{
					printf("Test String[118] failed: white space test of character %d!\n", i);
					failed = true;
				}
			}
			if (failed) (*failedTests)++;
			(*performedTests)++;
		}

		if (String::InstanceCount != 0) {printf("Test String[119] failed!\n"); (*failedTests)++;}
		(*performedTests)++;
	}
#endif

bool String::StartsWith(const char * string) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length1 = strlen(Data);
	unsigned int length2 = strlen(string);
	if (length2 > length1)
		return false;

	return strncmp(Data, string, length2) == 0;
}

bool String::StartsWithIC(const char * string) const
{
	ASSERTION_THIS;
	ASSERTION(string != NULL);

	unsigned int length1 = strlen(Data);
	unsigned int length2 = strlen(string);
	if (length2 > length1)
		return false;

	return strnicmp(Data, string, length2) == 0;
}

String String::Substring(unsigned int fromIndex, unsigned int length) const
{
	ASSERTION_THIS;

	unsigned int currentLength = strlen(Data);
	if (fromIndex > currentLength)
		return String();
	if (length > currentLength - fromIndex)
		length = currentLength - fromIndex;

	return String(Data, fromIndex, length);
}

String String::Substring(unsigned int fromIndex) const
{
	ASSERTION_THIS;

	if (fromIndex > strlen(Data))
		return String();

	return String(&Data[fromIndex]);
}

void String::ToLowerCase()
{
	ASSERTION_THIS;
	
	int length = strlen(Data);
	for (int i=0; i<length; i++)
		Data[i] = ToLowerCase(Data[i]);
}

unsigned char String::ToLowerCase(unsigned char character)
{
	if (character >= 'A' && character <= 'Z'
		|| character >= 192 && character <= 222 && character != 215)
		return character + 32;
	else
		return character;
}

void String::ToUpperCase()
{
	ASSERTION_THIS;
	
	int length = strlen(Data);
	for (int i=0; i<length; i++)
		Data[i] = ToUpperCase(Data[i]);
}

unsigned char String::ToUpperCase(unsigned char character)
{
	if (character >= 'a' && character <= 'z'
		|| character >= 224 && character != 247 && character != 255)
		return character - 32;
	else
		return character;
}

void String::Trim()
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);

	unsigned int left;
	for (left = 0; left < length && (Data[left] == ' '
		|| Data[left] == '\n' || Data[left] == '\r'
		|| Data[left] == '\t' || Data[left] == '\f'); left++);

	unsigned int right = length - 1;
	if (left < length)
		for (; right >= 0 && (Data[right] == ' ' || Data[right] == '\n' || Data[right] == '\r'
			|| Data[right] == '\t' || Data[right] == '\f'); right--);
	right++;

	memmove(Data, &Data[left], right - left);
	Data[right - left] = 0;
}

void String::TrimLeft()
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);

	unsigned int left;
	for (left = 0; left < length && (Data[left] == ' '
		|| Data[left] == '\n' || Data[left] == '\r'
		|| Data[left] == '\t' || Data[left] == '\f'); left++);

	memmove(Data, &Data[left], length - left);
	Data[length - left] = 0;
}

void String::TrimRight()
{
	ASSERTION_THIS;

	unsigned int length = strlen(Data);

	unsigned int right = length - 1;
	for (right = length - 1; right >= 0 &&
		(Data[right] == ' ' || Data[right] == '\n' || Data[right] == '\r'
		|| Data[right] == '\t' || Data[right] == '\f'); right--);
	right++;

	Data[right] = 0;
}

bool String::WriteTo(const char * fileName, bool textMode, bool append)
{
	ASSERTION_THIS;
	ASSERTION(fileName != NULL);
	ASSERTION(fileName[0] != 0);

	const char * mode = textMode ? (append ? "at" : "wt") : (append ? "ab" : "wb");

	FILE * file;
	if ((file = fopen(fileName, mode)) == NULL)
		return false;

	bool success = fputs(Data, file) >= 0;
	fclose(file);

	return success;
}
